'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = babelPluginTransformMetadata;

var _getTypesInfo = require('./visitors/getTypesInfo');

var _getTypesInfo2 = _interopRequireDefault(_getTypesInfo);

var _addReflections = require('./visitors/addReflections');

var _addReflections2 = _interopRequireDefault(_addReflections);

var _createTypeForAnnotation = require('./factories/createTypeForAnnotation');

var _createTypeForAnnotation2 = _interopRequireDefault(_createTypeForAnnotation);

var _createCreateCreateGenericTypeMetadata = require('./factories/createCreateCreateGenericTypeMetadata');

var _createCreateCreateGenericTypeMetadata2 = _interopRequireDefault(_createCreateCreateGenericTypeMetadata);

var _createCreateCreateObjectTypeMetadata = require('./factories/createCreateCreateObjectTypeMetadata');

var _createCreateCreateObjectTypeMetadata2 = _interopRequireDefault(_createCreateCreateObjectTypeMetadata);

var _createTypeForAnnotations = require('./factories/createTypeForAnnotations');

var _createTypeForAnnotations2 = _interopRequireDefault(_createTypeForAnnotations);

var _createGetUniqueTypeName = require('./factories/createGetUniqueTypeName');

var _createGetUniqueTypeName2 = _interopRequireDefault(_createGetUniqueTypeName);

var _createParentPathInsertAfter = require('./modifiers/createParentPathInsertAfter');

var _createParentPathInsertAfter2 = _interopRequireDefault(_createParentPathInsertAfter);

var _createReplaceMagicTypeCasts = require('./modifiers/createReplaceMagicTypeCasts');

var _createReplaceMagicTypeCasts2 = _interopRequireDefault(_createReplaceMagicTypeCasts);

var _createInjectParamTypes = require('./metaCreators/createInjectParamTypes');

var _createInjectParamTypes2 = _interopRequireDefault(_createInjectParamTypes);

var _createDetectDepIndexByNodeSuperClass = require('./factories/createDetectDepIndexByNodeSuperClass');

var _createDetectDepIndexByNodeSuperClass2 = _interopRequireDefault(_createDetectDepIndexByNodeSuperClass);

var _createInsertFactory = require('./factories/createInsertFactory');

var _createInsertFactory2 = _interopRequireDefault(_createInsertFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaults = {
    reflectImport: null,
    typeNameStrategy: 'typeName',
    paramKey: 'design:paramtypes',
    typeKey: 'design:function',
    onlyExports: false,
    ambiantTypeCastImport: 'babel-plugin-transform-metadata/_',
    ambiantDepsImport: 'babel-plugin-transform-metadata/Deps',
    reservedGenerics: ['Class', 'ResultOf'],
    jsxPragma: '__h',
    depsPositions: [{
        import: 'fake-react',
        name: 'Component',
        pos: 1
    }, {
        import: 'react',
        name: 'React|Component',
        pos: 2
    }]
};

function mapMasks(dp) {
    return _extends({}, dp, {
        name: new RegExp('^' + dp.name + '$', 'g')
    });
}

function babelPluginTransformMetadata(_ref) {
    var t = _ref.types;

    var cnf = void 0;
    return {
        visitor: {
            Program: function Program(path, _ref2) {
                var opts = _ref2.opts;

                if (!cnf) {
                    cnf = _extends({}, defaults, opts);
                    cnf.depsPositions = cnf.depsPositions.map(mapMasks);
                }
                var getUniqueTypeName = (0, _createGetUniqueTypeName2.default)(cnf.typeNameStrategy);
                var state = {
                    getUniqueTypeName: getUniqueTypeName,
                    reflectImport: cnf.reflectImport,
                    ambiantTypeCastImport: cnf.ambiantTypeCastImport,
                    ambiantDepsImport: cnf.ambiantDepsImport,

                    lastImportPath: null,
                    reservedGenerics: new Set(cnf.reservedGenerics),
                    injectId: null,
                    ambiantTypeCast: null,
                    imports: new Map(),
                    externalClassNames: new Map(),
                    internalTypes: new Map(),
                    externalTypeNames: new Map(),
                    exportNames: new Map(),
                    functionsWithJsx: new Set()
                };
                path.traverse(_getTypesInfo2.default, state);
                var replaceMagicTypeCasts = (0, _createReplaceMagicTypeCasts2.default)(t, state.externalTypeNames);
                var createCreateObjectTypeMetadata = (0, _createCreateCreateObjectTypeMetadata2.default)(t);
                var createCreateGenericTypeMetadata = (0, _createCreateCreateGenericTypeMetadata2.default)(t, state.externalTypeNames, state.internalTypes, state.reservedGenerics, state.externalClassNames);
                var typeForAnnotation = (0, _createTypeForAnnotation2.default)(t, state.externalTypeNames, createCreateObjectTypeMetadata, createCreateGenericTypeMetadata);
                var typeForAnnotations = (0, _createTypeForAnnotations2.default)(typeForAnnotation);

                var injectId = state.injectId;
                var injectorDeclaration = null;
                if (!injectId && cnf.reflectImport) {
                    injectId = t.identifier('Driver');
                    injectorDeclaration = t.importDeclaration([t.importDefaultSpecifier(injectId)], t.stringLiteral(cnf.reflectImport));
                }

                var defineParamTypes = (0, _createInjectParamTypes2.default)(t, injectId, typeForAnnotations, cnf.paramKey, cnf.typeKey);
                var parentPathInsertAfter = (0, _createParentPathInsertAfter2.default)(defineParamTypes);
                var reflectionState = {
                    t: t,
                    detectDepIndexByNodeSuperClass: (0, _createDetectDepIndexByNodeSuperClass2.default)(state.imports, cnf.depsPositions),
                    magicTypeCasts: [],
                    parentPaths: [],
                    onlyExports: cnf.onlyExports,
                    exportNames: state.exportNames,
                    magicTypeCastExpression: state.ambiantTypeCast ? state.ambiantTypeCast.node.specifiers[0].local.name : ''
                };

                path.traverse(_addReflections2.default, reflectionState);

                reflectionState.magicTypeCasts.forEach(replaceMagicTypeCasts);
                reflectionState.parentPaths.forEach(parentPathInsertAfter);
                if (cnf.jsxPragma) {
                    state.functionsWithJsx.forEach((0, _createInsertFactory2.default)(t, cnf.jsxPragma));
                }

                if (injectorDeclaration) {
                    if (state.lastImportPath) {
                        state.lastImportPath.insertAfter(injectorDeclaration);
                    } else {
                        path.node.body.unshift(injectorDeclaration);
                    }
                }

                if (state.ambiantTypeCast) {
                    state.ambiantTypeCast.remove();
                }
            }
        }
    };
}
//# sourceMappingURL=index.js.map