'use strict';

exports.__esModule = true;
var addReflections = {
    TypeCastExpression: function TypeCastExpression(path, _ref) {
        var magicTypeCasts = _ref.magicTypeCasts;
        var magicTypeCastExpression = _ref.magicTypeCastExpression;

        if (path.node.expression.name === magicTypeCastExpression) {
            magicTypeCasts.push(path);
        }
    },
    'ClassDeclaration|FunctionDeclaration': function ClassDeclarationFunctionDeclaration(path, _ref2) {
        var t = _ref2.t;
        var detectDepIndexByNodeSuperClass = _ref2.detectDepIndexByNodeSuperClass;
        var onlyExports = _ref2.onlyExports;
        var parentPaths = _ref2.parentPaths;
        var exportNames = _ref2.exportNames;

        var node = path.node;
        var parent = path.parent;
        var ref = node.id;
        if (onlyExports && !exportNames.has(ref.name)) {
            return;
        }
        var isInsertAfter = parent.type === 'ExportNamedDeclaration' || parent.type === 'ExportDefaultDeclaration';

        var params = void 0;

        if (!isInsertAfter && parent.type !== 'Program') {
            return;
        }

        if (node.type === 'ClassDeclaration') {
            if (node.superTypeParameters && node.superClass) {
                var sc = node.superClass;
                var depExtractIndex = detectDepIndexByNodeSuperClass(sc.name || sc.object.name);
                if (depExtractIndex !== null) {
                    var np = node.superTypeParameters.params;
                    if (np && np.length > depExtractIndex) {
                        var typedId = path.scope.generateUidIdentifier('deps');
                        typedId.typeAnnotation = np[depExtractIndex];
                        params = [t.typeAnnotation(typedId)];
                    }
                }
            }

            if (!params) {
                var body = node.body.body;
                for (var i = 0; i < body.length; i++) {
                    var bodyNode = body[i];
                    if (bodyNode.type === 'ClassMethod' && bodyNode.kind === 'constructor') {
                        params = bodyNode.params;
                        break;
                    } else if (bodyNode.type === 'ClassProperty') {
                        // @todo parameters from props
                    }
                }
            }
        } else {
            params = node.params;
        }

        if (params) {
            parentPaths.push([isInsertAfter ? path.parentPath : path, params, ref, node.type]);
        }
    }
};

exports.default = addReflections;
//# sourceMappingURL=addReflections.js.map