'use strict';

exports.__esModule = true;
var getTypesInfo = {
    ImportDeclaration: function ImportDeclaration(path, state) {
        /* eslint-disable no-param-reassign */
        var node = path.node;
        var strPath = node.source.value;
        if (strPath === state.reflectImport) {
            state.injectId = node.specifiers[0].local;
        }
        if (strPath === state.ambiantTypeCastImport) {
            state.ambiantTypeCast = path;
        } else {
            state.lastImportPath = path;
        }
    },
    'VariableDeclaration': function VariableDeclaration(path, state) {
        var node = path.node;
        var decls = node.declarations[0];
        state.externalClassNames.set(decls.id.name, decls.id.name);
    },
    'ClassDeclaration|FunctionDeclaration': function ClassDeclarationFunctionDeclaration(path, state) {
        var node = path.node;
        var pt = path.parent.type;
        if (pt === 'Program' || pt === 'ExportNamedDeclaration' || pt === 'ExportDefaultDeclaration') {
            state.externalClassNames.set(node.id.name, node.id.name);
        }
    },
    JSXElement: function JSXElement(path, state) {
        if (path.parent.type === 'JSXElement') {
            return;
        }

        var funcScope = path.scope;
        var found = false;
        while (!found) {
            funcScope = funcScope.getFunctionParent();
            var funcPath = funcScope.path;
            var funcNode = funcPath.node;
            if (funcPath.isProgram()) {
                break;
            }

            switch (funcNode.type) {
                case 'ClassMethod':
                    state.functionsWithJsx.add(funcNode);
                    found = true;
                    break;
                default:
                    if (funcPath.parent.type === 'Program') {
                        state.functionsWithJsx.add(funcNode);
                        found = true;
                    }
                    break;
            }
            funcScope = funcScope.parent;
        }
    },
    'ImportSpecifier|ImportDefaultSpecifier': function ImportSpecifierImportDefaultSpecifier(path, state) {
        var node = path.node;
        var parent = path.parent;
        var importedName = node.type === 'ImportDefaultSpecifier' ? node.local.name : node.imported.name;
        state.imports.set(node.local.name, { importedName: importedName, source: parent.source.value });
        state.externalClassNames.set(node.local.name, importedName);
        if (parent.importKind !== 'type') {
            return;
        }

        var token = state.getUniqueTypeName(parent.source.value, importedName);

        state.externalTypeNames.set(node.local.name, token);
    },
    ExportSpecifier: function ExportSpecifier(path, state) {
        var node = path.node;
        state.exportNames.set(node.local.name, node.exported.name);
    },
    'ExportNamedDeclaration|ExportDefaultDeclaration': function ExportNamedDeclarationExportDefaultDeclaration(path, state) {
        var node = path.node;
        var declaration = node.declaration;
        if (declaration) {
            var id = declaration.id || declaration;
            state.exportNames.set(id.name, id.name);
        }
    },
    'TypeAlias|InterfaceDeclaration': function TypeAliasInterfaceDeclaration(path, state) {
        var node = path.node;
        var parent = path.parent;
        state.externalClassNames.set(node.id.name, node.id.name);
        if (parent.type !== 'ExportNamedDeclaration') {
            state.internalTypes.set(node.id.name, node.type === 'TypeAlias' ? node.right : node.body);
            return;
        }
        var token = state.getUniqueTypeName('', node.id.name);

        state.externalTypeNames.set(node.id.name, token);
    }
};

exports.default = getTypesInfo;
//# sourceMappingURL=getTypesInfo.js.map